Crate spanley

Crate spanley 

Input

use spanley::Span;

let message = "\
  This is my generic string span, his name is Spanley.\n\
  Say hi Spanley!\
";

let spanley = Span::new(message, 57, 11).unwrap();

println!("{}", spanley);

Output

hi Spanley!

Modules

container ๐Ÿ”’
error ๐Ÿ”’
location ๐Ÿ”’
spanley ๐Ÿ”’

Structs

Span
A struct that spans over a &str, written as Span<'src>.
SpanContainer
A generic container to bundle a Span<'src> to an instance of type T it is representing. Written as SpanContainer<'src, T>
SpanLocation
A struct used to represent Spanโ€™s index field as line and character offset indices. Meant for logging purposes.

Enums

SpanError
The Error enum used by Span